import hashlib
import os

def file_hash(file_path: str, hash_method:str="sha256") -> str:
    if not os.path.isfile(file_path):
        return ''
    h = hashlib.new(hash_method)
    with open(file_path, 'rb') as f:
        while b := f.read(8192):
            h.update(b)
    return h.hexdigest()

# 自定义长度
def file_hash_len(file_path: str, hash_method:str, length: int) -> str:
    if not os.path.isfile(file_path):
        return ''
    h = hashlib.new(hash_method)
    with open(file_path, 'rb') as f:
        while b := f.read(8192):
            h.update(b)
    return h.hexdigest(length)


from . import hashfunc
from concurrent.futures import ProcessPoolExecutor
import math

def process_file(file_path, hash_method, length):
    if hash_method in ['shake_128', 'shake_256']:
        return {"file": file_path, "hash": hashfunc.file_hash_len(file_path, hash_method, length)}
    else:
        return {"file": file_path, "hash": hashfunc.file_hash(file_path, hash_method)}

def process_file_batch(file_paths, hash_method, length):
    return [process_file(file_path, hash_method, length) for file_path in file_paths]

def hash(f: list[str], hash_method: str = "sha256", length: int = 20, print_func=print) -> str:
    hlist = []
    lf = len(f)
    num_workers = ProcessPoolExecutor()._max_workers
    batch_size = math.ceil(lf / num_workers)
    file_batches = [f[i:i + batch_size] for i in range(0, lf, batch_size)]

    with ProcessPoolExecutor() as executor:
        futures = {executor.submit(process_file_batch, file_batch, hash_method, length): i for i, file_batch in enumerate(file_batches)}
        for future in futures:
            hlist.extend(future.result())

    return hlist


import http.client
import json
import haaaash

def get_latest_version():
    """获取 haaaash 的最新版本信息"""
    
    # 创建连接到 PyPI 的客户端
    conn = http.client.HTTPSConnection("pypi.org")

    # 发送请求获取 haaaash 包的版本信息
    conn.request("GET", "/pypi/haaaash/json")

    # 获取响应并解析
    response = conn.getresponse()
    data = response.read()
    package_info = json.loads(data)

    # 输出最新版本信息
    latest_version = package_info["info"]["version"]
    return latest_version

def main():
    latest_version = get_latest_version()
    """主函数"""
    print(rf"""______  __                                        ______  
___  / / /______ _______ _______ _______ ____________  /_ 
__  /_/ / _  __ `/_  __ `/_  __ `/_  __ `/__  ___/__  __ \
_  __  /  / /_/ / / /_/ / / /_/ / / /_/ / _(__  ) _  / / /
/_/ /_/   \__,_/  \__,_/  \__,_/  \__,_/  /____/  /_/ /_/ 

当前版本: {haaaash.__version__}
最新版本: {latest_version}

GitHub: https://github.com/gudupaospark/haaaash
Docs: https://haaaash.gudupao.top
""")
    if haaaash.__version__ != latest_version:
        print(f"\033[32m你应该更新 haaaash 到最新版本: {latest_version}，使用 \033[34mpip install haaaash --upgrade\033[32m 更新。\033[0m")
